/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.navigation
import androidx.navigation.compose.rememberNavController
import de.chaosdorf.meteroid.ui.home.DrinkListScreen
import de.chaosdorf.meteroid.ui.home.DrinkListViewModel
import de.chaosdorf.meteroid.ui.home.HomeSections
import de.chaosdorf.meteroid.ui.servers.AddServerScreen
import de.chaosdorf.meteroid.ui.servers.ServerListScreen
import de.chaosdorf.meteroid.ui.users.UserListScreen
import kotlinx.coroutines.launch

@Composable
fun AppRouter(viewModel: AppViewModel = viewModel()) {
  val scope = rememberCoroutineScope()
  val navController = rememberNavController()
  val initState by viewModel.initState.collectAsState()

  LaunchedEffect(initState) {
    when (initState) {
      AppViewModel.InitState.LOADING -> navController.navigate(Routes.Init)
      AppViewModel.InitState.CREATE_SERVER -> navController.navigate(Routes.Servers.Add)
      AppViewModel.InitState.SELECT_SERVER -> navController.navigate(Routes.Servers.List)
      AppViewModel.InitState.SELECT_USER -> navController.navigate(Routes.Users.List)
      AppViewModel.InitState.HOME -> navController.navigate(Routes.Home.Root)
    }
  }

  NavHost(navController, startDestination = Routes.Init) {
    composable(route = Routes.Init) { _ ->
      Box {
        Text("Loading")
      }
    }

    navigation(route = Routes.Servers.Root, startDestination = Routes.Servers.List) {
      composable(Routes.Servers.List) { _ ->
        ServerListScreen(
          hiltViewModel(),
          onAdd = { navController.navigate(Routes.Servers.Add) },
          onSelect = {
            scope.launch {
              viewModel.selectServer(it)
              navController.navigate(Routes.Users.List)
            }
          }
        )
      }
      composable(Routes.Servers.Add) { _ ->
        AddServerScreen(
          hiltViewModel(),
          onAdd = { navController.navigate(Routes.Servers.List) }
        )
      }
    }
    navigation(route = Routes.Users.Root, startDestination = Routes.Users.List) {
      composable(Routes.Users.List) { _ ->
        UserListScreen(
          hiltViewModel(),
          onAdd = { TODO() },
          onSelect = {
            scope.launch {
              viewModel.selectUser(it)
              navController.navigate(Routes.Home.Root)
            }
          },
          onBack = { navController.navigate(Routes.Servers.Root) },
        )
      }
      /*
      composable(Routes.Users.Add) { _ ->
        AddUserScreen(
          hiltViewModel(),
          onAdd = { navController.navigate(Routes.Users.List) }
        )
      }
       */
    }
    navigation(route = Routes.Home.Root, startDestination = Routes.Home.Purchase) {
      composable(Routes.Home.Purchase) { _ ->
        val drinkListViewModel = hiltViewModel<DrinkListViewModel>()
        MeteroidScaffold(
          routes = HomeSections.entries,
          currentRoute = HomeSections.PURCHASE,
          navigateTo = navController::navigate,
          onBack = { navController.navigate(Routes.Users.Root) },
        ) { paddingValues ->
          DrinkListScreen(drinkListViewModel, Modifier.padding(paddingValues))
        }
      }
      composable(Routes.Home.Deposit) { _ ->
        MeteroidScaffold(
          routes = HomeSections.entries,
          currentRoute = HomeSections.DEPOSIT,
          navigateTo = navController::navigate,
          onBack = { navController.navigate(Routes.Users.Root) },
        ) { paddingValues ->
          Box(Modifier.padding(paddingValues)) {
            Text("TODO: Deposit")
          }
        }
      }
    }
  }
}
